clear all;        % solve nonsymmetric nonlinear BVP y_xx + r(y,x) = 0, y(0)=y(1)=0
addpath('mn');    % only .mex routines used are OCset & OCLcoef (interpolation)
global fout symm geom meth Thx Rxvar Korder Kads
run OCCdefs             % script file with useful definitions
symm = NonSymmetric;
meth = Gauss;        % 1,2,3,4,5 for Gauss, Lobatto, Cheby, RadauR, RadauL
Rxvar = 1.0;         % source cubic value at left edge
Korder = 2;          % reaction order
Kads = 0.0;          % reaction denominator term
Thparm = 5.0;        % Thiele parameter, phi
Thfac = 2.0/sqrt((double(Korder)+1.0)*2.0);  % normalize, invalid for Kads > 0
Thparm = Thparm*Thfac;  Thx = 4.0*Thparm^2;
epsx = 1.0e-12;  mxiter = 25;       % convergence tolerance, maximum inerations
xi(:,1) = [0.0,0.01,0.02,0.03,0.05:0.05:0.95,0.97,0.98,0.99,1.0];  % for interpolated solution
nout = size(xi,2);   yi(1:nout) = 0.0;
fout = fopen('ex1n.dat','w');
fprintf(fout,'Thiele Modulus: actual %8.3f, factor %8.4f, & Variation Left %8.2f\n',Thparm,Thfac,Rxvar);
function [r,dr] = Rxn(y,x)
   global fout symm geom meth Korder Thx
   n = size(y);   % set px = Thx for constant coeff.
   px = Ex1_RxnPx(x);  % cubic variation
   [r,dr] = Ex1_RxnRate(y);
   r = Thx.*px'.*r;  dr = Thx.*px'.*dr;
end
ncalc = [4,6,2];  i = 0;   % interior points, counter
for n=ncalc(1):ncalc(3):ncalc(2)
   fprintf(fout,'Solution with %s points, n =%3i\n',strtrim(strmeth(meth,:)),n);
   fprintf(1,'Solution with %s points, n =%3i\n',strtrim(strmeth(meth,:)),n);
   i = i + 1;  nx = round((n+1)/2);
   [xc,wc,Ac] = OCnonsym(n,meth);               % points, quadrature wts., 1st derivative
   [Bc,Cc] = OCBCcoef(wc,Ac,Ac,NonSymmetric);   % Laplacian and stiffness
   y(1:n,1) = 0.0;
   for it=1:mxiter
      [r,dr] = Rxn(y,xc(2:n+1));
      A = Bc(2:n+1,2:n+1);                % nonsymmetric matrix
      b = A*y + r;  rnorm = norm(b,inf);  % residual & norm
      fprintf(1,'Iteration:\t%i\t%12.4e\t%10.6f\n',it,rnorm,y(nx));
      fprintf(fout,'Iteration:\t%i\t%12.4e\t%10.6f\n',it,rnorm,y(nx));
      if(rnorm < epsx),  break;  end
      A(1:n+1:n*n) = A(1:n+1:n*n) .+ dr'; % diagonal
      dely = -A\b;   y = y + dely;        % update
   end
   y = [0.0;y;0.0];   [r,dr] = Rxn(y,xc); % final values
   yi = polyval(polyfit(xc,y,n+1),xi);    % interpolated solution builtin functions
   ArrayPrintX(1,xc,y,0,'x, y - points, solution');
   ArrayPrintX(fout,xc,y,0,'x, y - points, solution');
   ArrayPrintX(fout,xi,yi,0,'x, y - interpolated solution solution');
   % flux calculations
   flux(1,i) = wc'*r;                                       % flux via integration
   flux(5,i) = Ac(1,:)*y;  flux(6,i) = -Ac(n+2,:)*y;        % left, right side derivative
   flux(4,i) = flux(5,i) + flux(6,i);                       % total flux from derivative
   flux(2,i) = flux(5,i) + wc(1).*(Bc(1,:)*y + r(1));       % residual correction
   flux(3,i) = flux(6,i) + wc(n+2).*(Bc(n+2,:)*y + r(n+2)); % residual correction
end
flux = flux/Thx;   % normalize
i = 0;
for n=ncalc(1):ncalc(3):ncalc(2)  % interior points
   i = i + 1;
   fprintf(fout,'%2i\t%12.8f\t%12.8f\t%12.8f\t%12.8f\t%12.8f\t%12.8f\n',n,flux(:,i));
   fprintf(1,'%2i\t%12.8f\t%12.8f\t%12.8f\t%12.8f\t%12.8f\t%12.8f\n',n,flux(:,i));
end
fclose(fout);
% simple plot of last solution
%plot(xc,y,'.','markersize',16)
%line(xi,yi), grid on
%title(sprintf('n = %i, %s, Flux(left,right) =%10.7f %10.7f',
%   n,strtrim(strmeth(meth,:)),flux(2,i),flux(3,i)))
