clear all;        % solve nonsymmetric BVP y_xx + r(y,x) = 0, y(0)=y(1)=0
addpath('mn');    % use .mex files for Gauss or Lobatto points
global fout symm geom meth Thx Rxvar Korder Kads
run OCCdefs       % define parameters, Geometry, etc.
xi(:,1) = [0.0,0.01,0.02,0.03,0.05:0.05:0.95,0.97,0.98,0.99,1.0];  % for interpolated solution
nout = size(xi,1);
% 'Exact' for comparison (n=12 with spline fit) - for cubic coefficient problem
xexact = [0,0.020032477,0.066099473,0.135565701,0.224680299,0.328637993,0.441834066, ...
          0.558165934,0.671362007,0.775319702,0.8644343,0.933900527,0.979967523,1];
yexact = [0,0.097511589,0.29484797,0.531037351,0.74240497,0.886060512,0.958470699, ...
          0.983554364,0.981022986,0.944803511,0.832325676,0.584843463,0.234695997,0];
yie = ppval(splinefit(xexact,yexact,11),xi);
titles = cellstr(['Conventional collocation matrix';    'Collocation, symmetric matrix:';
         'Galerkin or Moments, interpolated r:';  'Galerkin or Moments, exact integration:';]);
nsolv = 4; nflux = 7;  fout = 1;  fstd = 1; symm = NonSymmetric;
ncalc = [4,4];
Thparm = 5.0;  Rxvar = 0.2;  % can be read in by uncommenting below
disp('Diffusion/conduction problem: first order source w/ spatial variation');
% remove comments to supply values ----------------
%ch = input ( 'Enter Thiele modulus and reactivity on left side (0.2) >','s' );
%parm = sscanf(ch,'%f');  Thparm = parm(1);  Rxvar = parm(2);
% get Gauss or Lobatto -------------------
ch = input( 'Enter g/l/c/r/x for Gauss/Lobatto/Chebyshev/RadauR/RadauL >','s' );
cx = findstr(ch,'glcrx');   if(size(cx,1) > 0)meth = cx; end
ch = input ( 'Enter number interior points first and last >','s' );
ncalc = sscanf(ch,'%i');
ch = input ( 'Enter root of output file >','s' );
filenm = [ch '.dat'];  fout = fopen(filenm,'w');
Thx = 4.0*Thparm^2;
% --------------------------------------------------------
for n=ncalc(1):ncalc(2)
   fprintf(fout,'\nSolution using %s points with %2i interior points\n',strmeth(meth,:),n);
	fprintf(fout,'Thiele Modulus:\t%8.2f\t',Thparm);
	fprintf(fout,'Reaction variation:\t%8.4f\n',Rxvar);
   % --------------------------------------------------------
   nt = n + 2;  % total points
   ne = nt*nt;  % total matrix elements
   [xc,wc,Ac] = OCnonsym(n,meth);         % .m meth 1:5 = Lobatto, Gauss, Chebyshev, Radau R, Radau L
   [Bc,Cc] = OCBCcoef(wc,Ac,Ac,symm);     % .m Laplacian and stiffness
   Dc = MassMatrix(xc);
   Dm = MassMatrix(xc,@Ex1_RxnPx,2);         % MassFunc is a simple demo function
   Lc = Lcoef(xi,xc);
   % --------------------------------------------------------
   y(1:nt,1:nsolv) = 0.0;              % initialize solution
   yi(1:nout,1:nsolv) = 0.0;
   flux(1:nflux,1:nsolv) = 0.0;
   rx = (-Thx*Ex1_RxnPx(xc))';     % see Ex1_RxnPx above
   % --------------------------------------------------------
   % for these solutions, we construct the whole matrix (including
   % boundary points). Only the interior part is needed to solve for
   % y, but the boundary coefficients are needed for some flux
   % calculations
   for k=1:nsolv
      switch(k)
      case(1) % ----------------------------------------------
      % 1. Solution via conventional collocation
         A = Bc;
         r = rx;              % rate term RHS
         A(1:nt+1:ne) = A(1:nt+1:ne) + r';  % diagonal
      case(2) % ----------------------------------------------
      % 2. Symmetric solution matrix
         A = Cc;
         r = -wc.*rx;
         A(1:nt+1:ne) = A(1:nt+1:ne) + r';  %  diagonal
      case(3) % ----------------------------------------------
      % 3. Solution using mass matrix with interpolated rate terms
         r = -Dc*rx;
         A = Cc - Dc.*rx';    % broadcast rx across columns
      case(4) % ----------------------------------------------
      % 4. Exact calculation of mass matrix
         Dm = Thx*Dm;
         r = sum(Dm,2);
         A = Cc + Dm;
         ArrayPrintX(fout,Dm,0,0,'Mass Matrix:');
      end   % switch
      y(2:n+1,k) = A(2:n+1,2:n+1)\r(2:n+1);        % solve
      flux(:,k) = Ex1_Flux(rx,wc,Ac,A,r,y(:,k),k); % flux calc
      yi(:,k) = Lc*y(:,k);     % interpolate solution
      ArrayPrintX(fout,A,r,y(:,k),titles{k});
      ArrayPrint(fstd,y(:,k),titles{k});
   end   % for nsolv
   ArrayPrintX(fout,xc,y,0,'Solution at points: x, 4 solutions')
   ArrayPrintX(fout,xi,yie',yi,'Interpolated Solution: x, exact, 4 solutions');
   ArrayPrintX(fout,flux,0,0,'Flux Calculations');
end  % for ncalc
fclose(fout);
