clear all;        % solve symmetric nonlinear BVP y_xx + r(y) = 0, y' + Bi*y = 0 at x = 1
addpath('mn');    % only .mex routines used are OCset & OCLcoef (interpolation)
global fout symm geom meth Thx Rxvar Korder Kads
run OCCdefs       % script with useful definitions
fout = fopen('ex1s.dat','w'); fstd = 1;
epsx = 2.0e-13;  mxiter = 50;   % convergence tolerance, maximum iterations
geom = Planar;    % geometry defined in OCCdefs
geom = Spherical;
symm = Symmetric;
Bi = 100.0e12;       % Biot no. x3 Planar is equivalent spherical
Korder = 2;       % reaction order
Kads = 0.0; %0.95;      % denominator term (adsorption term)
if(Kads > 0.0 && Korder == 1)
  Thfac = sqrt(-2.0*(Kads + log(1.0-Kads)))/Kads;
elseif(Kads == 0.0)
  Thfac = 2.0/sqrt((double(Korder)+1.0)*2.0);
else
  Thfac = 1.0;    % could code others
end
Thfac = (geom+1)*Thfac; % phi and Bi x3 for spherical equivalence
BCres(1:3) = 0.0;  Res1(1:3) = 0.0; % B.C. residual, R(1)
xi = [0.0,0.01,0.02,0.03,0.05:0.05:0.95,0.97,0.98,0.99,1.0];  % interpolation points
nout = size(xi,2);  yi(1:nout,1:3) = 0.0;  calc(1:nout,1:2) = 0.0;
igx = 14;   xgx = xi(igx);  % use for guess
%Th1 = [0.65:0.10:0.85];  % copy vales to go forward and back
%nt = size(Th1,2);  Th(1:2*nt) = 0.0;  Th(1:nt) = Th1;  Th(nt+1:2*nt) = Th1(nt:-1:1);
Th = [2.0,5.0,8.0,10.0];  % forward pass only
nthp = size(Th,2);  fluxa(1:nthp,1:3) = 0.0;  % normalized fluxes
ncalc = [4,4,2];               % points to use
for n=ncalc(1):ncalc(3):ncalc(2)  % interior points
   n1 = n + 1;  rxx(1:n1,1:2) = 0.0;
   % code makes 3 attempts to solve using quadratic profile through these values
   ygx = [0.30,0.95,0.30];  % parameters for quadratic initial guess
   for ith=1:nthp    % loop over Thiele parameter (phi)
      Thparm = Th(ith); Thx = (Thparm*Thfac)^2;
      for meth=1:3   % solve by all 3 methods, Gauss, Lobatto, Chebyshev
         fprintf(1,'Solutions using %2i %s points, Thparm = %7.3f\n',n,
            strtrim(strmeth(meth,:)),Thparm);
         fprintf(fout,'\nResults: Thparm (general,actual) =%8.4f %8.4f, Bi =%10.3g\n',Thparm,Thparm*Thfac,Bi);
         fprintf(fout,'Source term: order =%6.2f, Ka =%6.2f\n',Korder,Kads);
         fprintf(fout,'Solutions using %2i %s points, %s geometry\n',n,
            strtrim(strmeth(meth,:)),strtrim(strgeom(geom+1,:)));
         [xc,wc,Ac,An] = OCsym(n,meth,geom);       % .m files
         [Bc,Cc] = OCBCcoef(wc,Ac,An,Symmetric);   % .m file
         wx = wc;    % copy to modified next stmt for boundary collocation
         %wx(n1) = 0.0;  Cc(n1,:) = Ac(n1,:); % remove comment for boundary collocation
         itx = 1; fac = ygx(1)/(1.0 - xgx^2); y = min(0.999,fac*(1.0 - xc.*xc));  % initial guess
         for it=1:mxiter   % Newton-Raphson iteration
            [r,dr] = Ex1_RxnRate(y);   % rate & derivative (function file)
            r = Thx.*r;   dr = Thx.*dr;
            Ax = Cc;  Ax(n1,n1) = Ax(n1,n1) + Bi;
            rx = wx.*r - Ax*y;         % residual of algebraic equation
            Ax(1:n1+1:n1*n1) = Ax(1:n1+1:n1*n1) - (wx.*dr)';  % diagonal
            rnorm = norm(rx,inf);      % residual norm
            fprintf(1,'Iteration:\t%i\t%12.4e\t%10.6f\n',it,rnorm,y(1));
            fprintf(fout,'Iteration:\t%i\t%12.4e\t%10.6f\n',it,rnorm,y(1));
            if(itx < 3 && (it > 10*itx && (rnorm > 0.01 | y(1) > 1.1 | y(1) < 0.01)))
               itx = itx + 1; fac = ygx(itx)/(1.0 - xgx^2);  y = min(0.999,fac*(1.0 - xc.*xc));
               fprintf(1,'Old/new guess =\t%10.6f\t%10.6f\n',ygx(itx-1),ygx(itx));
               fprintf(fout,'Old/new guess =\t%10.6f\t%10.6f\n',ygx(itx-1),ygx(itx));
             else
               y = y + Ax\rx;
             end
            if(rnorm < epsx),  break;  end
         end
         BCres(meth) = Bi*y(n1) + Ac(n1,:)*y;      % b.c. residual
         fluxa(ith,meth) = (geom+1)*(wc'*r)/Thx;   % normalized flux
         D2c = Bc*y;                               % 2nd derivative
         rxx(:,1) = r;  rxx(:,2) = D2c + r;        % rate & residual
         Res1(meth) = rxx(n1,2);
         xc2 = xc.*xc;  xi2 = xi.*xi;
         Lx = Lcoef(xi2',xc2);                     % interpolating poly.
         yi(:,meth) = Lx*y;
         calc(:,1) = Thx.*Ex1_RxnRate(yi(:,meth)); % rate at interpolation points
         calc(:,2) = Lx*D2c + calc(:,1);           % residual at interpolation points
         ArrayPrintX(fout,xc,y,rxx,'Solution:  x, y, r, resid.');
         ArrayPrintX(fout,xi',yi(:,meth),calc,'Interpolated x, y, Rate, Residual:');
         ygx(1) = 0.5 + (yi(igx,meth)-0.50)*1.10;  % spread slightly away from 0.5
         fprintf(fout,'New guess array:\t%12.8f at\t%12.8f\n',ygx(1),xgx);
      end
      fprintf(fout,'Flux:\t%20.16f\t%20.16f\t%20.16f\n',fluxa(ith,1),fluxa(ith,2),fluxa(ith,3));
      fprintf(fout,'BCresidual:\t%20.16f\t%20.16f\t%20.16f\n',BCres(1),BCres(2),BCres(3));
      fprintf(fout,'R1residual:\t%20.16f\t%20.16f\t%20.16f\n',Res1(1),Res1(2),Res1(3));
   end
   ArrayPrintX(fout,Th',fluxa,0,'Normalized Flux:');
end
fclose(fout);
      

