clear all;
addpath('mn');
global fout symm geom meth A wi
run OCCdefs    % define parameters, Symmetric, etc.
fout = fopen('ex4_ffs.dat','w');
Method = Lobatto;
n = 4;
zmax = 0.4;
zmethod = 3;
dz = 0.01;
Bcolloc = 0;   % 1 for boundary collocation
Nmethod = 12;
strODE = ['ode45 solver  ';'Forward Euler ';'Improved Euler';'Trapezoid Rule';'Backward Euler';
          '2nd Order Bkwd';'3rd Order Bkwd';'SDIRK 2nd Ord.';'EDIRK 2nd Ord.';'SDIRK3 3 stage';
          'EDIRK3 4 stage';'EDIRK3 5 stage'];
%ch = input( 'Enter g/l/c/r/x for Gauss/Lobatto/Chebyshev/RadauR/RadauL >','s' );
%cx = findstr(ch,'glcrx');   if(size(cx,1) > 0)Method = cx; end
disp('ODE Methods:');
for i=1:Nmethod
   fprintf(1,'  %3i. %s \n',i,strODE(i,:));
end
ch = input( 'Enter method no. >','s' );
zmethod = str2num(ch);
%ch = input ( 'Enter number interior points first, last, delta >','s' );
%ncalc = sscanf(ch,'%i');
fprintf(1,'%2i %s points, %s Method\n',n,strtrim(strmeth(Method,:)),strODE(zmethod,:));
fprintf(fout,'%2i %s points, %s Method\n',n,strtrim(strmeth(Method,:)),strODE(zmethod,:));
% ------------------------------------
function [yx,yc] = CalcFlux(y,wx,Cx)
   y2 = Cx(2,:)*y;      % last node
   yc = [0.0; y; y2];   % nodal values
   yx(1) = 1.5*(wx*y);  % average y
   yx(2) = -Cx(1,:)*y;  % flux
   yx(3) = yx(2)/yx(1); % Sh no.
end
% ------------------------------------
function yp = FuncFF(z,y)
global fout symm geom meth A wi
   yp = -(A*y).*wi;
end
% ------------------------------------
function [ys,yc] = BackDiff(yc0,A,wx,Cx,dz,nbk,nstep)  % 0,1,2 - 1st, 2nd, 3rd order
   bkx = [1.0, -1.0, 0.0, 0.0; 1.5, -2.0, 0.5, 0.0; 11.0/6.0, -18.0/6.0, 9.0/6.0, -2.0/6.0];
   n = size(A,1);  n1 = n + 1;
   y(1:n) = 1.0;   yc(1:nstep,1:n+2) = 0.0;  ys(1:nstep,1:3) = 0.0;
   ak = bkx(nbk+1,:)/dz;   ak(3:4) = ak(3:4)/ak(2);   wi = -wx*ak(2);
   Aw = A + diag(wx*ak(1));
   [L,U] = lu(Aw);   % LU factors
   for k = 1:nbk+1
      [ys(k,:),yc(k,:)] = CalcFlux(yc0(k,2:n1)',wx,Cx);
   end
   for k = nbk+2:nstep
      switch nbk
      case 0;   r = wi.*yc(k-1,2:n1);
      case 1;   r = wi.*(yc(k-1,2:n1) + ak(3)*yc(k-2,2:n1));
      case 2;   r = wi.*(yc(k-1,2:n1) + ak(3)*yc(k-2,2:n1) + ak(4)*yc(k-3,2:n1));
      end
     %y = Aw\r';
      y = U\(L\r');  % using LU factors
      [ys(k,:),yc(k,:)] = CalcFlux(y,wx,Cx);
   end
end
% ------------------------------------
function v = Cnorm(C,w)
   C = bsxfun(@times,((1.0)./w),C);
   v(1) = eigs(C,1);    % max eigenvalue
   v(2) = norm(C,1);    % 1 norm
   v(3) = norm(C,2);    % euclidian norm
   v(4) = norm(C,inf);  % infinite norm
   v(5) = norm(C,'fro');% frobenius norm
end
% ------------------------------------
nt = n + 2;  % total points
[xc,wc,Ac] = OCnonsym(n,Method);             % 1:5 = Lobatto, Gauss, Chebyshev, Radau R, Radau L
[Bc,Cc] = OCBCcoef(wc,Ac,Ac,NonSymmetric);   % Laplacian and stiffness
if(Bcolloc)
   Cc(nt,:) = Ac(nt,:);    % boundary collocation
end
A(1:n,1:n) = Cc(2:n+1,2:n+1) - Cc(2:n+1,nt).*Cc(nt,2:n+1)/Cc(nt,nt);
wx(1:n) = (1.0 - xc(2:n+1).*xc(2:n+1)).*wc(2:n+1);
Cx(2,:) = -Cc(nt,2:n+1)/Cc(nt,nt);           % coeff. y(n+1)
Cx(1,:) = Cc(1,2:n+1) + Cc(1,nt).*Cx(2,:);   % coeff. flux
vn = Cnorm(A,wx);
vn = (2.0)./vn;
ArrayPrint(fout,vn,'Maximum Euler stepsize and estimates:');
nstep = 1 + zmax/dz;
dzi = 1.0/dz;
wi(1:n,1) = (1.0)./wx;
wdz(1:n,1) = -dz*wi;
z(1:nstep,1) = [0.0:dz:zmax];
yc(1:nstep,1:nt) = 0.0;  ys(1:nstep,1:3) = 0.0;  y(1:n,1) = 1.0;
[ys(1,:),yc(1,:)] = CalcFlux(y,wx,Cx);
switch zmethod  % solution method
case 1  % built in solver ---------------------------------
   ode_opt = odeset('RelTol',0.1);  % doesn't seem to make a difference
   [z,yode] = ode45(@FuncFF,[0.0 zmax],y');
   tol1 = odeget(ode_opt,'RelTol');
   nstep = size(yode,1);
   for k=1:nstep
      [ys(k,:),yc(k,:)] = CalcFlux(yode(k,:)',wx,Cx);
   end
case 2  % Forward Euler -----------------------------------
   for k = 2:nstep
    % y = y .+ (A*y).*wdz;          % direct calc.
      y = y .+ dz*FuncFF(z(k),y);   % using function
      [ys(k,:),yc(k,:)] = CalcFlux(y,wx,Cx);
      if(abs(ys(k,2)) > 1.0e4)break; end
   end
case 3  % 2nd order Runge-Kutta Imp. Euler ----------------
   for k = 2:nstep
      f0 = (A*y).*wdz;  y = y + f0;
      f1 = (A*y).*wdz;  y = y + (f1 - f0)*0.5;
      [ys(k,:),yc(k,:)] = CalcFlux(y,wx,Cx);
      if(abs(ys(k,2)) > 1.0e4)break; end
   end
case 4  % Trapezoidal Rule --------------------------------
   Aw = A + diag(wx*(2.0*dzi));
   [L,U] = lu(Aw);   % LU factors
   for k = 2:nstep
      r = -2.0*(A*y);
     %y = y + Aw\r;     % factors every time
      y = y + U\(L\r);  % solution with factors
      [ys(k,:),yc(k,:)] = CalcFlux(y,wx,Cx);
   end
case {5,6,7} % 1st, 2nd & 3rd order backward diff. --------
   nbk = zmethod - 5;  % no. points to look back
   yc0(1,:) = yc(1,:);
   if(nbk > 0)
      mstart = [1,3];
      dirk = mstart(nbk); % use DIRK to start
      [ys0,yc0] = DIRK(A,wx,Cx,dz,dirk,nbk+1);
   end
   [ys,yc] = BackDiff(yc0,A,wx,Cx,dz,nbk,nstep);  % 0,1,2 - 1st, 2nd, 3rd order
case {8,9,10,11,12} % DIRK methods  -----------------------
   dirk = zmethod - 7;
   [ys,yc] = DIRK(A,wx,Cx,dz,dirk,nstep);
otherwise
   disp('bad selection')
end
ArrayPrintX(fout,z,ys,0,'z, yavg, flux, Sh:');
ArrayPrintX(fout,z,yc,0,'z, yc:');
fclose(fout);
