function [f] = Ex1_Flux(rc,wc,Ac,Ax,rs,y,isolv)
   global fout symm geom meth Thx Rxvar Korder Kads
   % This code performs flux calculations
   % rc - rate coefficient at xc
   % wc - quadratue weights
   % Ac - 1st derivative
   % Ax - solution matrix (from Bc or Cc see main)
   % rs - right-hand-side vector
   % y  - solution at points
   % isolv - the solution case 1:4
   % f(1:7) calculated flux as follows:
   % 1 - integral of rate for total flux
   % 2,3,4 - left, right, total natural treatment
   % 5,6,7 - left, right, total first derivative
   % derivatives ok for Gauss, bad for Lobatto
   % ----------------------------------
   % Other examples codes may be easier to follow
   % This one is complicated because of the 4 solution methods,
   % including Galerkin and Moments with a full mass matrix
   % check simpler codes first, e.g. ex1_non
   nt = size(y,1);
   f(1:7) = 0.0;
   f(2) = rs(1) - Ax(1,:)*y;
   f(3) = rs(nt) - Ax(nt,:)*y;
   % calculation from first derivative
   f(5) = Ac(1,:)*y; f(6) = -Ac(nt,:)*y;
   if(isolv == 1)  % conventional collocation formulation
      f(2) = f(5) - f(2)*wc(1);  % coefficients give boundary residual
      f(3) = f(6) - f(3)*wc(nt); % see case 1 in main
   end
   f(4) = f(2) + f(3);  f(7) = f(5) + f(6);
   f(1) = -((wc.*rc)')*(1.0 - y);  % total flux, material balance
   f = f/Thx;  % scale results
end

