function [x] = Jacobi_root(n,a,b)
%  if(a != -5)    % use this to force full calcs always
   if(a != b)     % use this for shortcut, full calcs only if a != b
      x = Jacobi_Xroot(n,a,b);
   elseif(n == 1)
      x(1:n,1) = 0.0;
   else  % shortcut for a=b, bb= -1/2 even, +1/2 odd
      n2 = fix(n/2);  n12 = fix((n+1)/2);  bb = -0.5 + (n12-n2);
      x2 = Jacobi_Xroot(n2,a,bb);
      x2 = sqrt((1.0+x2)*0.5);
      x(1:n,1) = 0.0;  x(n12+1:end) = x2;  x(n2:-1:1) = -x2;
   end
end
% ---- Jacobi polynomial root calculator
function [x] = Jacobi_Xroot(n,a,b)
   xtol = 1000.0*eps;   MaxNR = 8;
   p(1:n,1:n+1) = 0.0;  pp(1:n,1:n+1) = 0.0;
   ab = r_jacobi(n,a,b);            % recursion coefficients
   x(:,1) = RootEstimate(n,a,b);    % root estimates
   for i=1:MaxNR
      dp = Pcalc(x,ab,p,pp);
      x = x .- dp;
%     fprintf(1,'Jacobi_root = %4i\t%12.4e\n',i,max(abs(dp)));
      if(max(abs(dp)) < xtol)break; end
   end
end
% ---- Jacobi polynomial calculates p/p'
function dp = Pcalc(x,ab,p,pp)
   p(:,1)  = 1.0;  p(:,2)  = x(:,1) - ab(1,1);  pp(:,1) = 0.0;  pp(:,2) = 1.0;
   n = size(x,1);
   for k = 2:n
      p(:,k+1) = (x(:,1) - ab(k,1)).*p(:,k) - ab(k,2)*p(:,k-1);
      pp(:,k+1) = p(:,k) + (x(:,1) - ab(k,1)).*pp(:,k) - ab(k,2)*pp(:,k-1);
   end
   dp = p(:,n+1)./pp(:,n+1);
end
% ------  estimate roots using Gatteschi [Gautschi and Giordano (2008)]:
function x=RootEstimate(n,a,b)
   z2 = (2.0).*[1:n];  abn = 1.0/(2*n+a+b+1.0);
   ang0 = pi*(z2 + b - 0.5)*abn;  % x = -cos(ang0) % rough estimate
   z2 = tan(0.5*ang0);  ang0 = ang0 + ((0.25-b*b)./z2-(0.25-a*a).*z2).*(abn*abn);
   x = -cos(ang0);
   if(n < 3)return; end
   xb = RootEstBoundary(n,a,b);
   nb = size(xb,1);
   if(xb(1) > 0.0)
      x(n:-1:n-nb+1) = xb;
   end
   xb = RootEstBoundary(n,b,a);
   if(xb(1) > 0.0)
      x(1:nb) = -xb;
   end
end
% ---------- estimate roots near boundary -------------------------------
function x=RootEstBoundary(n,a,b)
  xjx = [2.404825557695772768622, 3.831705970207512315614; ....
         5.520078110286310649597, 7.015586669815618753537 ];
  j = round(a)+1; x(1:2,1) = 0.0; c45 = 1.0/45.0;
  if(abs(a-j+1) > 1.0e-6)return;  end
  d = 2.0*n + a + b + 1.0;
  v = 1.0/sqrt(d*d + (1.0 - a*a)/3.0 - b*b);
  r = (4.0-a*a-15.0*b*b)*(c45*v**4);
  ang = 2.0*xjx(:,j).*v.*(1.0 - r.*(0.5*xjx(:,j).^2+a*a-1.0));
  x = cos(ang);
end

