function L = Lcoef(x,xc)
global fout          % output file handle
   %! -------------------------------------------
   %!  Calculate values of the Lagrange interpolating polynomials:
   %!     x,xx   - points where values are desired (evaluation points)
   %!     xc,xxc - the n interpolation points (collocation points)
   %!     L(i,j) - value jth interpolating polynomial at xx(i)
   %!  example usage: given values yc at the n interpolation points
   %!     y(xx(:)) = MatMul(L,yc) is the interpolant at xx(:)
   %! -------------------------------------------
   n = size(x);   nc = size(xc);
   L(1:n,1:nc) = 0.0;  px(1:nc) = 0.0; pp(1:nc) = 0.0;
   xdif = xc'-xc+eye(nc,nc);  % eye(nc) gives col. vector (1,0,0,..)^t???
   wpp = (1.0)./prod(xdif,1);
%   ArrayPrint(1,x,'Lcoef x:');
%   ArrayPrint(1,xc,'Lcoef xc:');
%   ArrayPrint(fout,wpp,'weights:');
   for k=1:n
      if(min(abs(x(k)-xc(:))) > 0.0001)
         L(k,:) = prod(x(k)-xc)*(wpp./(x(k)-xc(:))');
      else
         for i=1:nc % if close to interpolation point, use the long calculation
            px(:) = x(k)-xc(:);  px(i) = 1.0;   pp(i) = prod(px);
         end
         L(k,:) = pp.*wpp;
      end
   end
%   ArrayPrint(1,L,'Lcoef L:');
end

