function [D] = MassMatrix(xc,func,nqx)
global fout symm geom meth         % output file handle
   % Mass matrix calculated with quadrature for moment or Galerkin
   % xc - collocation points
   % func - optional function of x in mass matrix
   % nqx additional quadrature points
   % ---------------------------------------------------
   Gauss = 1; Lobatto = 2;  Symmetric = 1; NonSymmetric = 0;
   nx = 1;  % default to one additional pt
   if(nargin == 3)nx = nqx; end
   if(meth > Lobatto)nx = 0;  end
   nt = numel(xc);   n = nt-2+symm;  nq = n + nx;
   icase = meth + 10*symm;
   if(symm == Symmetric)
      [xqq,wqq] = OCsym(nq,meth,geom);
   else
      [xqq,wqq] = OCnonsym(nq,meth);
   end
   if(nargin < 2)
      fqq(1:nt+nx,1) = 1.0;
   else
      fqq(1:nt+nx,1) = func(xqq);
   end
   wqq = wqq.*fqq;
   if(symm == Symmetric)               % symmetric problems
      xq = xqq(1:nq);   wq = wqq(1:nq);
      xc2 = xc.*xc;     xq2 = xq.*xq;
      Lq = Lcoef(xq2,xc2);
   else                                % nonsymmetric problems
      xq = xqq(2:nq+1);  wq = wqq(2:nq+1);
      Lq = Lcoef(xq,xc);
   end
   switch icase
   case (Gauss+10*NonSymmetric)
      wq = wq./(xq.*(1.0-xq)); % nonsymmetric, Moment modification
   case (Gauss+10*Symmetric)
      wq = wq./(1.0-xq2);      % symmetric, Moment modification
   end
   if(meth <= Lobatto)
      D = (Lq')*bsxfun(@times,wq,Lq);  % moments or Galerkin
   else
      D(1:nt,1:nt) = 0.0;
      D(1:nt+1:nt*nt) = wqq;           % others use diagonal
   end
   switch icase
   case(Gauss + 10*Symmetric)    % modify for Moments
      D(n+1,:) = 0.0;
      D(1:n,:) = bsxfun(@times,1.0 - xc2(1:n),D(1:n,:));
   case(Gauss + 10*NonSymmetric)
      D(1,:) = 0.0;  D(n+2,:) = 0.0;
      D(2:n+1,:) = bsxfun(@times,xc(2:n+1).*(1.0 - xc(2:n+1)),D(2:n+1,:));
   case(Lobatto + 10*Symmetric)  % Lobatto, contribution of boundary weights
      D(n+1,n+1) = D(n+1,n+1) + wqq(nq+1);
      wqq(nq+1)
   case(Lobatto + 10*NonSymmetric)
      D(1,1) = D(1,1) + wqq(1);
      D(n+2,n+2) = D(n+2,n+2) + wqq(nq+2);
   end
end
