function [xc,wc,A,An] = OCsymC(n,geom)
   % n interior points for Chebyshev points
   % geom = 0,1,2 for planar, cylindrical, spherical
   % xc - collocation points
   % wc - quadrature weights
   % A  - 1st derivative (even)
   % An - 1st derivative (odd)
   n1 = n + 1; nn2 = n + n + 1;  symm = 1;
   if(geom == 0)
      [x,w] = clencurt(nn2);             % Trefethen code
     %[x,w] = fclencurt(nn2+1,-1.0,1.0); % von Winckel FFT code
      xc(1:n1,1) = -x(n1+1:nn2+1);
      wc(1:n1,1) = w(n1+1:nn2+1);        % collect right end values
   else
      xc = -cos(pi*(n1:nn2)'/nn2);
      wc = Wpoly(xc,symm,geom);     % cyl & sphere integrate Lagrange directly
   end
   if(nargout < 3)return; end
   u = xc.*xc;  udif = u-u'+eye(n1);  dpx = prod(udif,2);   % use eye to avoid divide by 0
   A = dpx./(dpx'.*udif); A = 2.0*bsxfun(@times,xc,A);      % derivative operator
   A(1:n1+1:n1*n1) = (2.0).*xc - sum(A,2);                  % diagonal for zero row sum
   An = bsxfun(@rdivide,xc,xc').*A;                         % 1st der. of odd function 
   An(1:n1+1:n1*n1) = An(1:n1+1:n1*n1) + ((geom+1.0)./xc)'; % diagonal                 
end
